@@ -17,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: ser_win32.c,v 1.5 2004/07/15 17:29:35 troth Exp $ */
+/* $Id: ser_win32.c,v 1.7 2005/06/19 21:38:03 joerg_wunsch Exp $ */
 
 /*
  * Native Win32 serial interface for avrdude.
@@ -33,6 +33,8 @@
 extern char *progname;
 extern int verbose;
 
+long serial_recv_timeout = 5000; /* ms */
+
 #define W32SERBUFSIZE 1024
 
 struct baud_mapping {
@@ -70,7 +72,7 @@ static DWORD serial_baud_lookup(long baud)
 }
 
 
-BOOL serial_w32SetTimeOut(HANDLE hComPort, DWORD timeout) // in ms
+static BOOL serial_w32SetTimeOut(HANDLE hComPort, DWORD timeout) // in ms
 {
 	COMMTIMEOUTS ctmo;
 	ZeroMemory (&ctmo, sizeof(COMMTIMEOUTS));
@@ -81,14 +83,35 @@ BOOL serial_w32SetTimeOut(HANDLE hComPort, DWORD timeout) // in ms
 	return SetCommTimeouts(hComPort, &ctmo);
 }
 
-int serial_open(char * port, long baud)
+static int ser_setspeed(int fd, long baud)
 {
 	DCB dcb;
+	HANDLE hComPort = (HANDLE)fd;
+
+	ZeroMemory (&dcb, sizeof(DCB));
+	dcb.DCBlength = sizeof(DCB);
+	dcb.BaudRate = serial_baud_lookup (baud);
+	dcb.fBinary = 1;
+	dcb.fDtrControl = DTR_CONTROL_DISABLE;
+	dcb.fRtsControl = RTS_CONTROL_DISABLE;
+	dcb.ByteSize = 8;
+	dcb.Parity = NOPARITY;
+	dcb.StopBits = ONESTOPBIT;
+
+	if (!SetCommState(hComPort, &dcb))
+		return -1;
+
+	return 0;
+}
+
+
+static int ser_open(char * port, long baud)
+{
 	LPVOID lpMsgBuf;
 	HANDLE hComPort=INVALID_HANDLE_VALUE;
 
 	/* if (hComPort!=INVALID_HANDLE_VALUE) 
-		fprintf(stderr, "%s: serial_open(): \"%s\" is already open\n",
+		fprintf(stderr, "%s: ser_open(): \"%s\" is already open\n",
 				progname, port);
 	*/
 
@@ -106,7 +129,7 @@ int serial_open(char * port, long baud)
 			(LPTSTR) &lpMsgBuf,
 			0,
 			NULL);
-		fprintf(stderr, "%s: serial_open(): can't open device \"%s\": %s\n",
+		fprintf(stderr, "%s: ser_open(): can't open device \"%s\": %s\n",
 				progname, port, (char*)lpMsgBuf);
 		LocalFree( lpMsgBuf );
 		exit(1);
@@ -115,25 +138,16 @@ int serial_open(char * port, long baud)
 	if (!SetupComm(hComPort, W32SERBUFSIZE, W32SERBUFSIZE))
 	{
 		CloseHandle(hComPort);
-		fprintf(stderr, "%s: serial_open(): can't set buffers for \"%s\"\n",
+		fprintf(stderr, "%s: ser_open(): can't set buffers for \"%s\"\n",
 				progname, port);
 		exit(1);
 	}
 
-	ZeroMemory (&dcb, sizeof(DCB));
-	dcb.DCBlength = sizeof(DCB);
-	dcb.BaudRate = serial_baud_lookup (baud);
-	dcb.fBinary = 1;
-	dcb.fDtrControl = DTR_CONTROL_DISABLE;
-	dcb.fRtsControl = RTS_CONTROL_DISABLE;
-	dcb.ByteSize = 8;
-	dcb.Parity = NOPARITY;
-	dcb.StopBits = ONESTOPBIT;
 
-	if (!SetCommState(hComPort, &dcb))
+	if (ser_setspeed((int)hComPort, baud) != 0)
 	{
 		CloseHandle(hComPort);
-		fprintf(stderr, "%s: serial_open(): can't set com-state for \"%s\"\n",
+		fprintf(stderr, "%s: ser_open(): can't set com-state for \"%s\"\n",
 				progname, port);
 		exit(1);
 	}
@@ -141,7 +155,7 @@ int serial_open(char * port, long baud)
 	if (!serial_w32SetTimeOut(hComPort,0))
 	{
 		CloseHandle(hComPort);
-		fprintf(stderr, "%s: serial_open(): can't set initial timeout for \"%s\"\n",
+		fprintf(stderr, "%s: ser_open(): can't set initial timeout for \"%s\"\n",
 				progname, port);
 		exit(1);
 	}
@@ -150,7 +164,7 @@ int serial_open(char * port, long baud)
 }
 
 
-void serial_close(int fd)
+static void ser_close(int fd)
 {
 	HANDLE hComPort=(HANDLE)fd;
 	if (hComPort != INVALID_HANDLE_VALUE)
@@ -160,7 +174,7 @@ void serial_close(int fd)
 }
 
 
-int serial_send(int fd, char * buf, size_t buflen)
+static int ser_send(int fd, char * buf, size_t buflen)
 {
 	size_t len = buflen;
 	unsigned char c='\0';
@@ -169,7 +183,7 @@ int serial_send(int fd, char * buf, size_t buflen)
 	HANDLE hComPort=(HANDLE)fd;
 
 	if (hComPort == INVALID_HANDLE_VALUE) {
-		fprintf(stderr, "%s: serial_send(): port not open\n",
+		fprintf(stderr, "%s: ser_send(): port not open\n",
               progname); 
 		exit(1);
 	}
@@ -199,13 +213,13 @@ int serial_send(int fd, char * buf, size_t buflen)
 	serial_w32SetTimeOut(hComPort,500);
 
 	if (!WriteFile (hComPort, buf, buflen, &written, NULL)) {
-		fprintf(stderr, "%s: serial_send(): write error: %s\n",
+		fprintf(stderr, "%s: ser_send(): write error: %s\n",
               progname, "sorry no info avail"); // TODO
 		exit(1);
 	}
 
 	if (written != buflen) {
-		fprintf(stderr, "%s: serial_send(): size/send mismatch\n",
+		fprintf(stderr, "%s: ser_send(): size/send mismatch\n",
               progname); 
 		exit(1);
 	}
@@ -214,7 +228,7 @@ int serial_send(int fd, char * buf, size_t buflen)
 }
 
 
-int serial_recv(int fd, char * buf, size_t buflen)
+static int ser_recv(int fd, char * buf, size_t buflen)
 {
 	unsigned char c;
 	char * p = buf;
@@ -224,12 +238,12 @@ int serial_recv(int fd, char * buf, size_t buflen)
 	HANDLE hComPort=(HANDLE)fd;
 	
 	if (hComPort == INVALID_HANDLE_VALUE) {
-		fprintf(stderr, "%s: serial_read(): port not open\n",
+		fprintf(stderr, "%s: ser_read(): port not open\n",
               progname); 
 		exit(1);
 	}
 	
-	serial_w32SetTimeOut(hComPort,5000);
+	serial_w32SetTimeOut(hComPort, serial_recv_timeout);
 	
 	if (!ReadFile(hComPort, buf, buflen, &read, NULL)) {
 		LPVOID lpMsgBuf;
@@ -243,7 +257,7 @@ int serial_recv(int fd, char * buf, size_t buflen)
 			(LPTSTR) &lpMsgBuf,
 			0,
 			NULL 	);
-		fprintf(stderr, "%s: serial_recv(): read error: %s\n",
+		fprintf(stderr, "%s: ser_recv(): read error: %s\n",
 			      progname, (char*)lpMsgBuf);
 		LocalFree( lpMsgBuf );
 		exit(1);
@@ -274,7 +288,7 @@ int serial_recv(int fd, char * buf, size_t buflen)
 }
 
 
-int serial_drain(int fd, int display)
+static int ser_drain(int fd, int display)
 {
 	// int rc;
 	unsigned char buf[10];
@@ -284,7 +298,7 @@ int serial_drain(int fd, int display)
 	HANDLE hComPort=(HANDLE)fd;
 
   	if (hComPort == INVALID_HANDLE_VALUE) {
-		fprintf(stderr, "%s: serial_drain(): port not open\n",
+		fprintf(stderr, "%s: ser_drain(): port not open\n",
               progname); 
 		exit(1);
 	}
@@ -309,7 +323,7 @@ int serial_drain(int fd, int display)
 				(LPTSTR) &lpMsgBuf,
 				0,
 				NULL 	);
-			fprintf(stderr, "%s: serial_drain(): read error: %s\n",
+			fprintf(stderr, "%s: ser_drain(): read error: %s\n",
 					  progname, (char*)lpMsgBuf);
 			LocalFree( lpMsgBuf );
 			exit(1);
@@ -326,4 +340,16 @@ int serial_drain(int fd, int display)
   return 0;
 }
 
+struct serial_device serial_serdev =
+{
+  .open = ser_open,
+  .setspeed = ser_setspeed,
+  .close = ser_close,
+  .send = ser_send,
+  .recv = ser_recv,
+  .drain = ser_drain,
+};
+
+struct serial_device *serdev = &serial_serdev;
+
 #endif /* WIN32NATIVE */
